@@ -1,6 +1,4 @@
-#' replace elements of a list
-#'
-#' @param x,y named lists
+#' Merge two lists
 #'
 #' @keywords internal
 merge_list <- function(x, y) {
@@ -9,9 +7,8 @@ merge_list <- function(x, y) {
 }
 
 
-#' paths and uris
-#'
-#' @param uri a character, the path to a file in URI format
+#' Paths and uris
+#' @keywords internal
 path_from_uri <- function(uri) {
     if (is.null(uri)) {
         return(NULL)
@@ -21,7 +18,7 @@ path_from_uri <- function(uri) {
 }
 
 
-#' @param path a character, the path to a file
+#' @keywords internal
 #' @rdname path_from_uri
 path_to_uri <- function(path) {
     if (is.null(path)) {
@@ -31,27 +28,20 @@ path_to_uri <- function(path) {
     paste0(prefix, utils::URLencode(path))
 }
 
-#' check if a file is an RMarkdown file
-#'
-#' @template uri
-#'
-#' @return a logical
+#' Check if a file is an RMarkdown file
+#' @keywords internal
 is_rmarkdown <- function(uri) {
     filename <- path_from_uri(uri)
     endsWith(tolower(filename), ".rmd") || endsWith(tolower(filename), ".rmarkdown")
 }
 
-#' check if a token is in a R code block in an Rmarkdown file
+#' Check if a token is in a R code block in an Rmarkdown file
 #'
 #' In an RMarkdown document, tokens can be either inside an R code block or
 #' in the text. This function will return `FALSE` if the token is in the text
-#' and `TRUE` if it is in a code block. For R scripts, it always returns `TRUE`.
+#' and `TRUE` if it is in a code block. For any other files, it always returns `TRUE`.
 #'
-#' @template uri
-#' @template document
-#' @template position
-#'
-#' @return a logical
+#' @keywords internal
 check_scope <- function(uri, document, position) {
     if (is_rmarkdown(uri)) {
         line <- position$line
@@ -63,17 +53,14 @@ check_scope <- function(uri, document, position) {
 }
 
 
-#' calculate character offset based on the protocol
-#'
-#' @param s a character / character vector
-#'
+#' Calculate character offset based on the protocol
 #' @keywords internal
 ncodeunit <- function(s) {
     lengths(iconv(s, from = "UTF-8", to = "UTF-16BE", toRaw = TRUE)) / 2
 }
 
 
-#' determinal code units given code points
+#' Determinal code units given code points
 #'
 #' @param line a character of text
 #' @param pts 0-indexed code points
@@ -89,36 +76,18 @@ code_point_to_unit <- function(line, pts) {
 }
 
 
-#' determinal utf16 code points given utf8 code points
-#'
-#' @param line a character of text
-#' @param pts 0-indexed code points
-#'
+#' Check if a path is a directory
 #' @keywords internal
-utf8_to_utf16_code_point <- function(line, pts) {
-    utf16cp <- c(0, cumsum(ncodeunit(strsplit(line, "")[[1]])))
-    utf8cp <- c(0, cumsum(nchar(strsplit(line, "")[[1]], type = "bytes")))
-    utf16cp[match(pts, utf8cp)]
-}
-
-
-#' check if a filename is a directory
-#'
-#' @param filename a character
-#'
-#' @keywords internal
-is_directory <- function(filename) {
-    is_dir <- file.info(filename)$isdir
+is_directory <- function(path) {
+    is_dir <- file.info(path)$isdir
     !is.na(is_dir) && is_dir
 }
 
-#' find the root package folder
+#' Find the root package folder
 #'
 #' This function searches backwards in the folder structure until it finds
 #' a DESCRIPTION file or it reaches the top-level directory.
 #'
-#' @param path a character
-#'
 #' @keywords internal
 find_package <- function(path = getwd()) {
     start_path <- getwd()
@@ -149,8 +118,6 @@ is_package <- function(rootUri) {
 
 #' read a character from stdin
 #'
-#' @param n an integer
-#'
 #' @keywords internal
 stdin_read_char <- function(n) {
     .Call("stdin_read_char", PACKAGE = "languageserver", n)
@@ -163,11 +130,11 @@ stdin_read_line <- function() {
     .Call("stdin_read_line", PACKAGE = "languageserver")
 }
 
-#' check if the current process become an orphan
+#' check if the current process becomes an orphan
 #'
 #' @keywords internal
-become_orphan <- function() {
-    .Call("become_orphan", PACKAGE = "languageserver")
+process_is_detached <- function() {
+    .Call("process_is_detached", PACKAGE = "languageserver")
 }
 
 #' throttle a function execution
@@ -193,8 +160,6 @@ throttle <- function(fun, t = 1) {
 #'
 #' Remove unwanted objects, _e.g._ `names<-`, `%>%`, etc.
 #'
-#' @param objects a character vector
-#'
 #' @keywords internal
 sanitize_names <- function(objects) {
     objects[stringr::str_detect(objects, "^(?:[^\\W_]|\\.)(?:[^\\W]|\\.)*$")]
@@ -212,7 +177,7 @@ look_forward <- function(text) {
 
 look_backward <- function(text) {
     matches <- stringr::str_match(
-        text, "\\b(?<!\\$)(?:([a-zA-Z][a-zA-Z0-9.]+)(:::?))?((?:[^\\W_]|\\.)(?:[^\\W]|\\.)*)?$")
+        text, "(?<!\\$)(?:\\b|(?=\\.))(?:([a-zA-Z][a-zA-Z0-9.]+)(:::?))?((?:[^\\W_]|\\.)(?:[^\\W]|\\.)*)?$")
     list(
         full_token = na_to_empty_string(matches[1]),
         package  = na_to_empty_string(matches[2]),
