@@ -3,42 +3,30 @@
 #' A `Workspace` is initialized at the start of a session, when the language
 #' server is started. Its goal is to contain the `Namespace`s of the packages
 #' that are loaded during the session for quick reference.
-#'
-#' @section Methods:
-#' + `load_package(pkgname)`: add a new `Namespace` for `pkgname` if necessary
-#' and if possible
-#' + `guess_package(object)`: returns the most recently loaded package in which
-#' `object` can be found
-#' + `get_namespace(pkgname)`:  return the [Namespace] of `pkgname`
-#' + `get_signature(funct, pkgname = NULL)`: return the signature of `funct`
-#' + `get_formals(funct, pkgname = NULL)`: return the [base::formals()] of `funct`
-#' + `get_help(topic, pkgname = NULL)`: return the help text of `topic`
-#' + `load_to_global(parse_result)`:
-#'
-#' @field pkgname a character, a package name
-#' @field object a character, an object name
-#' @field funct a character, a function name
-#' @field topic a character, a help topic name
-#' @field parse_result ?
+#' @keywords internal
 Workspace <- R6::R6Class("Workspace",
-    public = list(
-        loaded_packages = c("base", "stats", "methods", "utils", "graphics", "grDevices"),
-        namespaces = list(),
+    private = list(
+        parse_data = list(),
         global_env = list(nonfuncts = character(0),
                           functs = character(0),
                           signatures = list(),
-                          formals = list(),
-                          definitions = DefinitionCache$new()),
+                          formals = list()),
+        namespaces = list(),
+        definitions = NULL
+    ),
+    public = list(
+        loaded_packages = c("base", "stats", "methods", "utils", "graphics", "grDevices"),
 
         initialize = function() {
             for (pkgname in self$loaded_packages) {
-                self$namespaces[[pkgname]] <- Namespace$new(pkgname)
+                private$namespaces[[pkgname]] <- Namespace$new(pkgname)
             }
+            private$definitions <- DefinitionCache$new()
         },
 
         load_package = function(pkgname) {
             if (!(pkgname %in% self$loaded_packages)) {
-                ns <- tryCatch(self$get_namespace(pkgname), error = function(e) NULL)
+                ns <- self$get_namespace(pkgname)
                 logger$info("ns: ", ns)
                 if (!is.null(ns)) {
                     self$loaded_packages <- append(self$loaded_packages, pkgname)
@@ -52,7 +40,7 @@ Workspace <- R6::R6Class("Workspace",
 
             for (pkgname in rev(self$loaded_packages)) {
                 ns <- self$get_namespace(pkgname)
-                if (ns$exists(object)) {
+                if (!is.null(ns) && ns$exists(object)) {
                     return(pkgname)
                 }
             }
@@ -61,51 +49,48 @@ Workspace <- R6::R6Class("Workspace",
 
         get_namespace = function(pkgname) {
             if (pkgname == "_workspace_") {
-                self$global_env
-            } else if (pkgname %in% names(self$namespaces)) {
-                self$namespaces[[pkgname]]
+                private$global_env
+            } else if (pkgname %in% names(private$namespaces)) {
+                private$namespaces[[pkgname]]
+            } else if (length(find.package(pkgname, quiet = TRUE))) {
+                private$namespaces[[pkgname]] <- Namespace$new(pkgname)
+                private$namespaces[[pkgname]]
             } else {
-                self$namespaces[[pkgname]] <- Namespace$new(pkgname)
-                self$namespaces[[pkgname]]
+                NULL
             }
         },
 
         get_signature = function(funct, pkgname = NULL) {
             if (is.null(pkgname)) {
-                if (funct %in% self$global_env$functs) {
-                    return(self$global_env$signatures[[funct]])
+                if (funct %in% private$global_env$functs) {
+                    return(private$global_env$signatures[[funct]])
                 }
                 pkgname <- self$guess_package(funct)
             }
             if (is.null(pkgname)) {
                 NULL
             } else {
-                tryCatch({
-                    ns <- self$get_namespace(pkgname)
+                ns <- self$get_namespace(pkgname)
+                if (!is.null(ns)) {
                     ns$get_signature(funct)
-                    },
-                    error = function(e) NULL
-                )
+                }
             }
-
         },
 
         get_formals = function(funct, pkgname = NULL) {
             if (is.null(pkgname)) {
-                if (funct %in% self$global_env$functs) {
-                    return(self$global_env$formals[[funct]])
+                if (funct %in% private$global_env$functs) {
+                    return(private$global_env$formals[[funct]])
                 }
                 pkgname <- self$guess_package(funct)
             }
             if (is.null(pkgname)) {
                 NULL
             } else {
-                tryCatch({
-                        ns <- self$get_namespace(pkgname)
-                        ns$get_formals(funct)
-                    },
-                    error = function(e) list()
-                )
+                ns <- self$get_namespace(pkgname)
+                if (!is.null(ns)) {
+                    ns$get_formals(funct)
+                }
             }
         },
 
@@ -128,15 +113,15 @@ Workspace <- R6::R6Class("Workspace",
         },
 
         get_definition = function(topic) {
-            self$global_env$definitions$get(topic)
+            private$definitions$get(topic)
         },
 
         get_definitions_for_uri = function(uri) {
-            self$global_env$definitions$get_functs_for_uri(uri)
+            private$definitions$get_functs_for_uri(uri)
         },
 
         get_definitions_for_query = function(query) {
-            self$global_env$definitions$filter(query)
+            private$definitions$filter(query)
         },
 
         get_code = function(topic, pkg = NULL) {
@@ -146,32 +131,39 @@ Workspace <- R6::R6Class("Workspace",
             }
             logger$info("pkg:", pkg)
 
-            if (is.null(pkg)) {
-                return(NULL)
-            } else {
+            if (!is.null(pkg) && length(find.package(pkg, quiet = TRUE))) {
                 code <- utils::getFromNamespace(topic, pkg)
-            }
-            if (length(code) > 0) {
-                code <- repr::repr_text(code)
-                # reorganize the code
-                code <- stringr::str_split(code, "\n")[[1]]
-                code[1] <- paste(topic, "<-", code[1])
-                enc2utf8(code[!grepl("<bytecode|<environment", code)])
-            } else {
-                NULL
+                if (length(code) > 0) {
+                    code <- repr::repr_text(code)
+                    # reorganize the code
+                    code <- stringr::str_split(code, "\n")[[1]]
+                    code[1] <- paste(topic, "<-", code[1])
+                    enc2utf8(code[!grepl("<bytecode|<environment", code)])
+                }
             }
         },
 
-        load_to_global = function(parse_result, uri) {
-            self$global_env$nonfuncts <- unique(
-                c(self$global_env$nonfuncts, parse_result$nonfuncts))
-            self$global_env$functs <- unique(
-                c(self$global_env$functs, parse_result$functs))
-            self$global_env$signatures <- merge_list(
-                self$global_env$signatures, parse_result$signatures)
-            self$global_env$formals <- merge_list(
-                self$global_env$formals, parse_result$formals)
-            self$global_env$definitions$update(uri, parse_result$definition_ranges)
+        get_parse_data = function(uri) {
+            private$parse_data[[uri]]
+        },
+
+        parse_file = function(uri, parse_data) {
+            if (!is.null(parse_data$xml_file) && 
+                file.exists(parse_data$xml_file)) {
+                parse_data$xml_doc <- xml2::read_xml(parse_data$xml_file)
+                file.remove(parse_data$xml_file)
+            }
+
+            private$parse_data[[uri]] <- parse_data
+            private$global_env$nonfuncts <- unique(
+                c(private$global_env$nonfuncts, parse_data$nonfuncts))
+            private$global_env$functs <- unique(
+                c(private$global_env$functs, parse_data$functs))
+            private$global_env$signatures <- merge_list(
+                private$global_env$signatures, parse_data$signatures)
+            private$global_env$formals <- merge_list(
+                private$global_env$formals, parse_data$formals)
+            private$definitions$update(uri, parse_data$definition_ranges)
         }
     )
 )
@@ -181,11 +173,13 @@ Workspace <- R6::R6Class("Workspace",
 #'
 #' internal use only
 #' @param uri the file uri
+#' @param temp_dir the temporary directory to store intermediate files
 #' @param temp_file the file to lint, determine from \code{uri} if \code{NULL}
 #' @param run_lintr set \code{FALSE} to disable lintr diagnostics
 #' @param parse set \code{FALSE} to disable parsing file
+#' @param resolve set \code{FALSE} to disable resolving package dependencies
 #' @export
-workspace_sync <- function(uri, temp_file = NULL, run_lintr = TRUE, parse = FALSE) {
+workspace_sync <- function(uri, temp_dir = NULL, temp_file = NULL, run_lintr = TRUE, parse = FALSE, resolve = FALSE) {
     if (is.null(temp_file)) {
         path <- path_from_uri(uri)
     } else {
@@ -193,10 +187,13 @@ workspace_sync <- function(uri, temp_file = NULL, run_lintr = TRUE, parse = FALS
     }
 
     if (parse) {
-        parse_result <- tryCatch(parse_document(path), error = function(e) NULL)
-        # parse_result <- parse_document(path)
+        parse_data <- tryCatch(parse_document(path, temp_dir), error = function(e) NULL)
+        # parse_data <- parse_document(path)
+        if (resolve) {
+            parse_data$packages <- resolve_package_dependencies(parse_data$packages)
+        }
     } else {
-        parse_result <- NULL
+        parse_data <- NULL
     }
 
     if (run_lintr) {
@@ -206,7 +203,7 @@ workspace_sync <- function(uri, temp_file = NULL, run_lintr = TRUE, parse = FALS
         diagnostics <- NULL
     }
 
-    list(parse_result = parse_result, diagnostics = diagnostics)
+    list(parse_data = parse_data, diagnostics = diagnostics)
 }
 
 
@@ -225,7 +222,9 @@ process_sync_in <- function(self) {
             item <- sync_out$pop(uri)
             process <- item$process
             parse <- item$parse
-            if (process$is_alive()) try(process$kill(), silent = TRUE)
+            if (process$is_alive()) {
+                tryCatch(process$kill(), error = function(e) NULL)
+            }
             temp_file <- item$temp_file
             if (!is.null(temp_file) && file.exists(temp_file)) {
                 file.remove(temp_file)
@@ -235,6 +234,7 @@ process_sync_in <- function(self) {
         item <- sync_in$pop(uri)
         run_lintr <- item$run_lintr && self$run_lintr
         parse <- parse || item$parse
+        resolve <- item$resolve
         doc <- item$document
         path <- path_from_uri(uri)
         if (is.null(doc)) {
@@ -251,9 +251,11 @@ process_sync_in <- function(self) {
                     function(...) languageserver::workspace_sync(...),
                     list(
                         uri = uri,
+                        temp_dir = tempdir(),
                         temp_file = temp_file,
                         run_lintr = run_lintr,
-                        parse = parse
+                        parse = parse,
+                        resolve = resolve
                     ),
                     system_profile = TRUE, user_profile = TRUE
                 ),
@@ -283,14 +285,14 @@ process_sync_out <- function(self) {
                     )
                 )
             }
-            parse_result <- process_result$parse_result
-            if (!is.null(parse_result)) {
-                for (package in parse_result$packages) {
+            parse_data <- process_result$parse_data
+            if (!is.null(parse_data)) {
+                for (package in parse_data$packages) {
                     logger$info("load package:", package)
                     self$workspace$load_package(package)
                 }
 
-                self$workspace$load_to_global(parse_result, uri)
+                self$workspace$parse_file(uri, parse_data)
             }
 
             # cleanup
